@@ -8,6 +8,7 @@ from clip import Clip
 from working_directory import Tag
 from samimport import Import
 from samshrimp import Shrimp
+from bowtie import Bowtie
 from samconsensus import Filter, Reconsensus, Consensus
 from nway_diff import Nway
 from fisher_diff import Fisher
@@ -16,13 +17,14 @@ from samcount import Count
 from runr import Test_counts, Plot_counts, Heatmap, Compare_tests, Norm_from_counts, NMF
 from trivia import Test, As_fasta, As_gff, Sample, Stats
 from shred import Shred
-from igv import Make_genome, IGV_plots, As_userplots 
+from igv import Make_genome, IGV_plots, As_userplots, Run_igv
 from workflows import Analyse_sample
+from variant import Freebayes, Vcf_filter, Vcf_patch, Test_variant_call, Power_variant_call
 
 from legion import *
 
 
-VERSION='0.86'
+VERSION='0.87'
 
 BOLD = '\x1b[1m'
 END = '\x1b[m'
@@ -40,11 +42,14 @@ Give <tool>: without further arguments for help on using that tool.
 
     make-reference:
                   - Set up a directory containing a reference sequence,
-                    annotations, and files for SHRiMP.
+                    annotations, and files for SHRiMP and/or Bowtie.
 
     shrimp:       - Run SHRiMP 2 on a read set to set up a working
                     directory.
     
+    bowtie:       - Run Bowtie 2 on a read set to set up a working
+                    directory.
+    
     consensus     - Filter read hits, and try to call a consensus for each 
                     position in reference.
     
@@ -52,6 +57,28 @@ Give <tool>: without further arguments for help on using that tool.
     (filter:      - Filter read hits, but do not call consensus)
     (reconsensus: - Re-call consensus, using previously filtered hits)
 
+%(BOLD)sAlignment to reference -- VCF based tools: (under development)%(END)s
+
+These provide an alternative to consensus calling using "nesoni consensus:"
+- better handling of complicated Multi-Nucleotide Polymorphisms
+- can't distinguish between absence of a variant and insufficient data
+  (but can distinguish absence of a variant from insufficient data 
+   in a single sample if variant present in other samples)
+
+    freebayes:    - Run FreeBayes to produce a VCF file.
+    
+    vcf-filter:   - Filter a VCF file, eg as produced by "nesoni freebayes:".
+    
+    vcf-patch:    - Patch in variants to produce genome of samples.
+                    (similar to consensus_masked.fa produced by "nesoni consensus:")
+    
+    test-variant-call:
+                  - Generate synthetic reads, see what variant is called.
+    
+    power-variant-call:
+                  - Apply "neosni test-variant-call:" to a variety of
+                    different variants over a range of depths.
+
 %(BOLD)sAlignment to reference -- analysis tools:%(END)s
 
     igv-plots:    - Generate plots for IGV.
@@ -130,6 +157,8 @@ samcount, for analysis with BioConductor packages.
     
     make-genome:  - Make an IGV .genome file.
     
+    run-igv:      - Run IGV with a specified .genome file.
+    
     sample:       - Randomly sample from a sequence file.
     
     stats:        - Show some statistics about a sequence or annotation file.
@@ -156,7 +185,6 @@ behaviour:
 - annotation files can be in GENBANK or GFF format 
   (GFF is not yet supported by all tools).
 - nesoni is able to read files compressed with gzip or bzip2.
-- remote files can be specified as a URL (will be streamed using lftp).
 
 """ % { 'BOLD' : '\x1b[1m', 'END' : '\x1b[m', 'VERSION' : VERSION, 'MAKE' : Make().describe('', show_help=True) }
 
