@@ -1,11 +1,11 @@
 
-import os
+import sys, os
 
 from nesoni import io, grace, config, annotation
 
 class Reference(io.Workspace):
     def __init__(self, working_dir, must_exist):
-        super(Reference,self).__init__(working_dir, must_exist)        
+        super(Reference,self).__init__(working_dir, must_exist)
     
     def set_sequences(self, filenames):        
         reference_genbank_filename = self / 'reference.gbk'
@@ -60,16 +60,32 @@ class Reference(io.Workspace):
     def reference_fasta_filename(self):
         return self.object_filename('reference.fa')
 
-    def build_shrimp_mmap(self, cs=False):
+    def build_shrimp_mmap(self, cs=False, log_to=sys.stdout):
         suffix = '-cs' if cs else '-ls'
         
         grace.status('Building SHRiMP mmap')
-        io.execute([
-            'gmapper' + suffix,
-            '--save', self.object_filename('reference' + suffix),
-            self.reference_fasta_filename(),
-        ])
+        io.execute( [
+                'gmapper' + suffix,
+                '--save', self.object_filename('reference' + suffix),
+                self.reference_fasta_filename(),            
+                ],
+            stdout=log_to
+            )
         grace.status('')
+
+    def build_bowtie_index(self, log_to=sys.stdout):
+        io.execute([
+                'bowtie2-build',
+                self.reference_fasta_filename(),
+                self/'bowtie',
+                ],
+            stdout = log_to,
+            )
+    
+    def get_bowtie_index_prefix(self):
+        assert os.path.exists(self/'bowtie.1.bt2'), 'bowtie2 index was not created. Please use "make-reference:" with "--bowtie yes".'
+        return self/'bowtie'
+    
     
     def shrimp_command(self, cs=False, parameters = [ ]):
         """ Parameters:
@@ -110,10 +126,12 @@ and links to annotations.
 """)
 @config.Bool_flag('ls', 'Generate gmapper-ls mmap (faster SHRiMP startup for base-space reads).')
 @config.Bool_flag('cs', 'Generate gmapper-cs mmap (faster SHRiMP startup for color-space reads).')
+@config.Bool_flag('bowtie', 'Generate bowtie2 index (necessary in order to use "nesoni bowtie:").')
 @config.Main_section('filenames', 'Sequence and annotation files.')
 class Make_reference(config.Action_with_output_dir):
     ls = False
     cs = False
+    bowtie = False
     filenames = [ ]
 
     def run(self):
@@ -130,14 +148,20 @@ class Make_reference(config.Action_with_output_dir):
             if not any:
                 raise grace.Error(filename + ' is neither a sequence file nor an annotation file that nesoni can read.')
         
+        if not sequences:
+            raise grace.Error('No reference sequence files given.')
+        
         reference = Reference(self.output_dir, must_exist=False)        
         reference.set_sequences(sequences)
         reference.set_annotations(annotations)
-        if self.ls:
-            reference.build_shrimp_mmap(False)
-        if self.cs:
-            reference.build_shrimp_mmap(True)
         
+        with open(self.log_filename(),'wb') as f:
+            if self.ls:
+                reference.build_shrimp_mmap(False, f)
+            if self.cs:
+                reference.build_shrimp_mmap(True, f)
+            if self.bowtie:
+                reference.build_bowtie_index(f)
             
         
         
