@@ -3,74 +3,6 @@ from nesoni import io, bio, grace, sam, config, working_directory
 
 import os, sys, subprocess
 
-#
-#def cutoff_interpreter(cutoff_string):
-#    if cutoff_string.endswith('%'):
-#        mult = float(cutoff_string[:-1]) / 100.0 * 10.0
-#        return lambda length: length*mult
-#    else:
-#        value = int(cutoff_string)
-#        return lambda length: value
-#
-
-
-SHRIMP_HELP = """\
-Usage:
-
-    nesoni samshrimp: output_directory [options] \\
-        reference.fa/.gbk [...] \\
-        [reads: single.fq | single.fa [...]] \\
-        [interleaved: interleaved.fq | interleaved.fa [...]] \\
-        [pairs: left.fq right.fq | left.fa right.fa] \\
-        [shrimp-options: ...options to pass directly to gmapper-ls... ]
-  
-Paired end reads should either be given as two files in the "pairs" section, 
-or as a single interleaved file in the "inerleaved" section. 
-
-Files may be gzipped. Reference files may be in FASTA or GENBANK format.
-
-Options:
-
-    --cs yes/no            - Colorspace reads (i.e. use gmapper-cs).
-                             Default: no
-
-    --sam-unaligned yes/no - Pass --sam-unaligned to gmapper
-                             Default: yes
-    
-    --half-paired yes/no   - Pass --half-paired to gmapper
-                             Default: yes
-    
-To increase sensitivity, shrimp is by default invoked with the options:
-
-  -n 2 -w 200% --half-paired --sam-unaligned
-
-By default pairs are assumed to have:
-
-  -p opp-in -I 0,500
-  
-You can override these in the shrimp-options section.
-
-Useful shrimp options:
-  
-  -h threshold% - Reduce the hit threshold
-                  (default is 55% of read length)
-
-  -n 1          - For single reads: perform alignment on a single 
-                  seed hit (default 2) 
-                  (more sensitive, but slower)
-
-  -p, -I        - Pairing parameters, see SHRiMP documentation
-  
-  --qv-offset   - Quality offset.
-                  If not specified, nesoni will attempt to guess.
-                  For Sanger format fastq and recent Illumina, use 33.
-
-"""
-
-
-
-
-
 
 @config.help("""\
 Align reads to a reference using SHRiMP.
@@ -103,7 +35,9 @@ Useful shrimp options:
                   If not specified, nesoni will attempt to guess.
                   For Sanger format fastq and recent Illumina, use 33.
 """)
-@config.Main_section('references', 'Reference sequence filenames. Giving a GENBANK file will allow smarter downstream processing.')
+@config.Main_section('references', 
+    'Reference sequence filenames, '
+    'or a directory created using "nesoni make-reference: --bowtie yes" (recommended).')
 @config.Bool_flag('cs', 'Are reads in colorspace?')
 @config.Bool_flag('sam_unaligned', 'Pass --sam-unaligned to gmapper?')
 @config.Bool_flag('half_paired', 'Pass --half-paired to gmapper?')
@@ -140,9 +74,27 @@ class Shrimp(config.Action_with_output_dir):
             read_sets.append( (item, True) )
         for item in self.interleaved:
             read_sets.append( ([item], True) )
+
+        #Create working directory
         
-        default_options = { '-E' : None, '-T' : None, '-N' : str(grace.how_many_cpus()), '-n':'2', '-w':'200%',
-                            '-p': 'opp-in', '-I': '0,500', '-X':None }
+        workspace = self.get_workspace() #working_directory.Working(self.output_dir, must_exist=False)
+        workspace.setup_reference(self.references)        
+        workspace.update_param(snp_cost=25)
+        reference = workspace.get_reference()
+        reference_filename = reference.reference_fasta_filename()
+
+
+        
+        default_options = { 
+            '-E' : None, 
+            '-T' : None, 
+            '-N' : str(grace.how_many_cpus()), 
+            '-n':'2', 
+            '-w':'200%',
+            '-p': 'opp-in', 
+            '-I': '0,500', 
+            '-X':None,
+        }
         
         if self.sam_unaligned:
             default_options['--sam-unaligned'] = None
@@ -156,14 +108,7 @@ class Shrimp(config.Action_with_output_dir):
         cutoff = '55%' #Default changed in SHRiMP 2.0.2
         if '-h' in self.shrimp_options:
             cutoff = self.shrimp_options[ self.shrimp_options.index('-h')+1 ]
-        
-        #Create working directory
-        
-        workspace = self.get_workspace() #working_directory.Working(self.output_dir, must_exist=False)
-        workspace.setup_reference(self.references)        
-        reference = workspace.get_reference()
-        reference_filename = reference.reference_fasta_filename()
-        
+                
         #workspace = io.Workspace(self.output_dir)
         #
         #workspace.update_param( 
@@ -271,7 +216,7 @@ class Shrimp(config.Action_with_output_dir):
             assert len(set(guesses)) == 1, 'Conflicting quality offset guesses, please specify --qv-offset manually.'
             default_options['--qv-offset'] = str(guesses[0])
                 
-        for filenames, is_paired in read_sets:
+        for i, (filenames, is_paired) in enumerate(read_sets):
             options = self.shrimp_options[:]
                
             has_qualities = all(
@@ -308,6 +253,11 @@ class Shrimp(config.Action_with_output_dir):
             else:
                 reads_parameters = [ '-1', filenames[0], '-2', filenames[1] ]
             
+            default_options['--read-group'] = '%s-%d,%s' % (
+                workspace.name.replace(',','_'),
+                i+1,
+                workspace.name.replace(',','_')
+            )
             for flag in default_options:
                 if flag not in options:
                     options.append(flag)
